@@ -1,13 +1,8 @@
-use indoc::indoc;
-
-use assert_cmd::assert::OutputAssertExt;
-use predicates::str::PredicateStrExt;
-
-use crate::helpers::{project::binary, project_builder::tmp_dir};
+use crate::helpers::prelude::*;
 
 #[test]
 fn it_runs_all_hook_types() {
-    let template = tmp_dir()
+    let template = tempdir()
         .file(
             "init-script.rhai",
             indoc! {r#"
@@ -59,14 +54,11 @@ fn it_runs_all_hook_types() {
         .init_git()
         .build();
 
-    let dir = tmp_dir().build();
+    let dir = tempdir().build();
 
     binary()
-        .arg("gen")
-        .arg("--git")
-        .arg(template.path())
-        .arg("-n")
-        .arg("script-project")
+        .arg_git(template.path())
+        .arg_name("script-project")
         .arg("-d")
         .arg("pre=hello")
         .arg("-d")
@@ -89,7 +81,7 @@ fn it_runs_all_hook_types() {
 
 #[test]
 fn it_runs_system_commands() {
-    let template = tmp_dir()
+    let template = tempdir()
         .file(
             "system-script.rhai",
             indoc! {r#"
@@ -106,14 +98,11 @@ fn it_runs_system_commands() {
         .init_git()
         .build();
 
-    let dir = tmp_dir().build();
+    let dir = tempdir().build();
 
     binary()
-        .arg("gen")
-        .arg("--git")
-        .arg(template.path())
-        .arg("-n")
-        .arg("script-project")
+        .arg_git(template.path())
+        .arg_name("script-project")
         .arg("--allow-commands")
         .current_dir(dir.path())
         .assert()
@@ -125,7 +114,7 @@ fn it_runs_system_commands() {
 
 #[test]
 fn it_fails_to_prompt_for_system_commands_in_silent_mode() {
-    let template = tmp_dir()
+    let template = tempdir()
         .file(
             "system-script.rhai",
             indoc! {r#"
@@ -142,14 +131,11 @@ fn it_fails_to_prompt_for_system_commands_in_silent_mode() {
         .init_git()
         .build();
 
-    let dir = tmp_dir().build();
+    let dir = tempdir().build();
 
     binary()
-        .arg("gen")
-        .arg("--git")
-        .arg(template.path())
-        .arg("-n")
-        .arg("script-project")
+        .arg_git(template.path())
+        .arg_name("script-project")
         .arg("--silent")
         .current_dir(dir.path())
         .assert()
@@ -160,46 +146,50 @@ fn it_fails_to_prompt_for_system_commands_in_silent_mode() {
 
 #[test]
 fn it_fails_when_a_system_command_returns_non_zero_exit_code() {
-    let template = tmp_dir()
+    let template = tempdir()
         .file(
             "system-script.rhai",
-            indoc! {r#"
-                let output = system::command("mkdir", ["invalid_/.dir_name"]);
-            "#},
+            r#"let output = system::command("mkdir", ["invalid_/.dir_name"]);"#,
         )
         .file(
             "cargo-generate.toml",
             indoc! {r#"
-            [hooks]
-            post = ["system-script.rhai"]
+                [hooks]
+                post = ["system-script.rhai"]
             "#},
         )
         .init_git()
         .build();
 
-    let dir = tmp_dir().build();
+    let dir = tempdir().build();
 
     binary()
-        .arg("gen")
-        .arg("--git")
-        .arg(template.path())
-        .arg("-n")
-        .arg("script-project")
+        .arg_git(template.path())
+        .arg_name("script-project")
         .arg("--allow-commands")
         .current_dir(dir.path())
         .assert()
         .failure()
         .stderr(
-            predicates::str::contains(
-                "System command `mkdir invalid_/.dir_name` returned non-zero status",
-            )
-            .from_utf8(),
+            if cfg!(target_os = "linux") {
+                predicates::str::contains(
+                    "System command `mkdir invalid_/.dir_name` failed to execute: mkdir: cannot create directory ‘invalid_/.dir_name’: No such file or directory"
+                ).from_utf8()
+            } else if cfg!(target_os = "macos") {
+                predicates::str::contains(
+                    "System command `mkdir invalid_/.dir_name` failed to execute: mkdir: invalid_: No such file or directory"
+                ).from_utf8()
+            } else { // Windows
+                predicates::str::contains(
+                    "System command `mkdir invalid_/.dir_name` failed to execute: The syntax of the command is incorrect."
+                ).from_utf8()
+            }
         );
 }
 
 #[test]
 fn it_fails_when_it_cant_execute_system_command() {
-    let template = tmp_dir()
+    let template = tempdir()
         .file(
             "system-script.rhai",
             indoc! {r#"
@@ -209,36 +199,36 @@ fn it_fails_when_it_cant_execute_system_command() {
         .file(
             "cargo-generate.toml",
             indoc! {r#"
-            [hooks]
-            post = ["system-script.rhai"]
+                [hooks]
+                post = ["system-script.rhai"]
             "#},
         )
         .init_git()
         .build();
 
-    let dir = tmp_dir().build();
+    let dir = tempdir().build();
 
     binary()
-        .arg("gen")
-        .arg("--git")
-        .arg(template.path())
-        .arg("-n")
-        .arg("script-project")
+        .arg_git(template.path())
+        .arg_name("script-project")
         .arg("--allow-commands")
         .current_dir(dir.path())
         .assert()
         .failure()
         .stderr(
-            predicates::str::contains(
-                "System command `dummy_command_that_doesn't_exist dummy_arg` failed to execute",
-            )
+            // TODO: This error message is different on MacOS and Linux. We should unify it.
+            predicates::str::contains(if cfg!(target_os = "macos") {
+                "System command `dummy_command_that_doesn't_exist dummy_arg` failed to execute"
+            } else {
+                "Failed executing script: system-script.rhai"
+            })
             .from_utf8(),
         );
 }
 
 #[test]
 fn it_can_change_case() {
-    let template = tmp_dir()
+    let template = tempdir()
         .file(
             "pre-script.rhai",
             indoc! {r#"
@@ -262,14 +252,11 @@ fn it_can_change_case() {
         .init_git()
         .build();
 
-    let dir = tmp_dir().build();
+    let dir = tempdir().build();
 
     binary()
-        .arg("gen")
-        .arg("--git")
-        .arg(template.path())
-        .arg("-n")
-        .arg("script-project")
+        .arg_git(template.path())
+        .arg_name("script-project")
         .current_dir(dir.path())
         .assert()
         .success()
@@ -285,10 +272,12 @@ fn it_can_change_case() {
 
 #[test]
 fn can_change_variables_from_pre_hook() {
-    let template = tmp_dir()
+    let template = tempdir()
         .file(
             "cargo-generate.toml",
             indoc! {r#"
+            [placeholders]
+            multi = {type = "array", prompt="??", choices=["a","b","c"], default=["a","b"]}
             [hooks]
             pre = ["pre-script.rhai"]
             "#},
@@ -297,37 +286,40 @@ fn can_change_variables_from_pre_hook() {
             "pre-script.rhai",
             indoc! {r#"
                 variable::set("foo", "bar");
+                variable::set("multi", ["Q","b"]);
             "#},
         )
         .file(
             "PRE-TEST",
             indoc! {r#"
                 {{foo}};
+                {{multi}};
             "#},
         )
         .init_git()
         .build();
 
-    let dir = tmp_dir().build();
+    let dir = tempdir().build();
 
     binary()
-        .arg("gen")
-        .arg("--git")
-        .arg(template.path())
-        .arg("-n")
-        .arg("script-project")
+        .arg_git(template.path())
+        .arg_name("script-project")
         .current_dir(dir.path())
+        .arg("-d")
+        .arg("multi=a,b")
         .assert()
         .success()
         .stdout(predicates::str::contains("Done!").from_utf8());
 
     assert!(dir.exists("script-project/PRE-TEST"));
-    assert!(dir.read("script-project/PRE-TEST").contains("bar"));
+    let pre_test = dir.read("script-project/PRE-TEST");
+    assert!(pre_test.contains("bar"));
+    assert!(pre_test.contains("Q"));
 }
 
 #[test]
 fn init_hook_can_set_project_name() {
-    let template = tmp_dir()
+    let template = tempdir()
         .file(
             "init.rhai",
             indoc! {r#"
@@ -350,12 +342,10 @@ fn init_hook_can_set_project_name() {
         .init_git()
         .build();
 
-    let dir = tmp_dir().build();
+    let dir = tempdir().build();
 
     binary()
-        .arg("gen")
-        .arg("--git")
-        .arg(template.path())
+        .arg_git(template.path())
         .current_dir(dir.path())
         .assert()
         .success()
@@ -369,7 +359,7 @@ fn init_hook_can_set_project_name() {
 
 #[test]
 fn init_hook_can_change_project_name_but_keeps_cli_name_for_destination() {
-    let template = tmp_dir()
+    let template = tempdir()
         .file(
             "init.rhai",
             indoc! {r#"
@@ -379,27 +369,24 @@ fn init_hook_can_change_project_name_but_keeps_cli_name_for_destination() {
         .file(
             "cargo-generate.toml",
             indoc! {r#"
-            [hooks]
-            init = ["init.rhai"]
+                [hooks]
+                init = ["init.rhai"]
             "#},
         )
         .file(
             "generated.txt",
             indoc! {r#"
-            {{crate_name}}
-        "#},
+                {{crate_name}}
+            "#},
         )
         .init_git()
         .build();
 
-    let dir = tmp_dir().build();
+    let dir = tempdir().build();
 
     binary()
-        .arg("gen")
-        .arg("--git")
-        .arg(template.path())
-        .arg("--name")
-        .arg("foo")
+        .arg_git(template.path())
+        .arg_name("foo")
         .current_dir(dir.path())
         .assert()
         .success()
@@ -411,7 +398,7 @@ fn init_hook_can_change_project_name_but_keeps_cli_name_for_destination() {
 
 #[test]
 fn init_hook_can_change_project_name_but_keeps_init_destination() {
-    let template = tmp_dir()
+    let template = tempdir()
         .file(
             "init.rhai",
             indoc! {r#"
@@ -434,15 +421,12 @@ fn init_hook_can_change_project_name_but_keeps_init_destination() {
         .init_git()
         .build();
 
-    let dir = tmp_dir().build();
+    let dir = tempdir().build();
 
     binary()
-        .arg("gen")
-        .arg("--git")
-        .arg(template.path())
-        .arg("--name")
-        .arg("foo")
-        .arg("--init")
+        .arg_git(template.path())
+        .arg_name("foo")
+        .flag_init()
         .current_dir(dir.path())
         .assert()
         .success()
@@ -454,7 +438,7 @@ fn init_hook_can_change_project_name_but_keeps_init_destination() {
 
 #[test]
 fn rhai_filter_invokes_rhai_script() {
-    let template = tmp_dir()
+    let template = tempdir()
         .file(
             "filter-script.rhai",
             indoc! {r#"
@@ -470,14 +454,11 @@ fn rhai_filter_invokes_rhai_script() {
         .init_git()
         .build();
 
-    let dir = tmp_dir().build();
+    let dir = tempdir().build();
 
     binary()
-        .arg("gen")
-        .arg("--git")
-        .arg(template.path())
-        .arg("-n")
-        .arg("filter-project")
+        .arg_git(template.path())
+        .arg_name("filter-project")
         .current_dir(dir.path())
         .assert()
         .success()
@@ -488,9 +469,57 @@ fn rhai_filter_invokes_rhai_script() {
         .contains("content from RHAI"));
 }
 
+#[test]
+fn date_works() {
+    let template = tempdir()
+        .file(
+            "init.rhai",
+            indoc! {r#"
+                let dt = system::date();
+                variable::set("year", `${dt.year}`);
+                variable::set("month", `${dt.month}`);
+                variable::set("day", `${dt.day}`);
+            "#},
+        )
+        .file(
+            "cargo-generate.toml",
+            indoc! {r#"
+                [hooks]
+                init = ["init.rhai"]
+            "#},
+        )
+        .file(
+            "generated.txt",
+            indoc! {r#"
+                {{year}}-{{month}}-{{day}}
+        "#},
+        )
+        .init_git()
+        .build();
+    let date = time::OffsetDateTime::now_utc();
+    let dir = tempdir().build();
+
+    binary()
+        .arg_git(template.path())
+        .arg_name("foo")
+        .flag_init()
+        .current_dir(dir.path())
+        .assert()
+        .success()
+        .stdout(predicates::str::contains("Done!").from_utf8());
+
+    assert!(dir.exists("generated.txt"), "generated.txt didn't exist");
+    let content = dir.read("generated.txt");
+    let expected = format!("{}-{}-{}", date.year(), u8::from(date.month()), date.day());
+    assert!(
+        content.contains(&expected),
+        "generated.txt didn't include `{expected}`:\n`{content}`"
+    );
+}
+
 #[test]
 fn missing_rhai_filter_fails_prints_warnings() {
-    let template = tmp_dir()
+    let template = tempdir()
         .file(
             "file_to_expand.txt",
             indoc! {r#"
@@ -500,14 +529,11 @@ fn missing_rhai_filter_fails_prints_warnings() {
         .init_git()
         .build();
 
-    let dir = tmp_dir().build();
+    let dir = tempdir().build();
 
     binary()
-        .arg("gen")
-        .arg("--git")
-        .arg(template.path())
-        .arg("-n")
-        .arg("filter-project")
+        .arg_git(template.path())
+        .arg_name("filter-project")
         .current_dir(dir.path())
         .assert()
         .success()
